@@ -90,7 +90,7 @@ class Superproject(object):
     self._git_event_log = git_event_log
     self._quiet = quiet
     self._print_messages = print_messages
-    self._branch = self._GetBranch()
+    self._branch = manifest.branch
     self._repodir = os.path.abspath(repodir)
     self._superproject_dir = superproject_dir
     self._superproject_path = os.path.join(self._repodir, superproject_dir)
@@ -100,6 +100,7 @@ class Superproject(object):
     if self._manifest.superproject:
       remote = self._manifest.superproject['remote']
       git_name = hashlib.md5(remote.name.encode('utf8')).hexdigest() + '-'
+      self._branch = self._manifest.superproject['revision']
       self._remote_url = remote.url
     else:
       self._remote_url = None
@@ -116,17 +117,6 @@ class Superproject(object):
     """Returns the manifest path if the path exists or None."""
     return self._manifest_path if os.path.exists(self._manifest_path) else None
 
-  def _GetBranch(self):
-    """Returns the branch name for getting the approved manifest."""
-    p = self._manifest.manifestProject
-    b = p.GetBranch(p.CurrentBranch)
-    if not b:
-      return None
-    branch = b.merge
-    if branch and branch.startswith(R_HEADS):
-      branch = branch[len(R_HEADS):]
-    return branch
-
   def _LogMessage(self, message):
     """Logs message to stderr and _git_event_log."""
     if self._print_messages:
@@ -381,7 +371,7 @@ def _UseSuperprojectFromConfiguration():
         print(('You are not currently enrolled in Git submodules experiment '
                '(go/android-submodules-quickstart).  Use --use-superproject '
                'to override.\n'), file=sys.stderr)
-    return user_value
+      return user_value
 
   # We don't have an unexpired choice, ask for one.
   system_cfg = RepoConfig.ForSystem()
@@ -420,4 +410,6 @@ def UseSuperproject(opt, manifest):
     if client_value is not None:
       return client_value
     else:
+      if not manifest.superproject:
+        return False
       return _UseSuperprojectFromConfiguration()
